type StorageIsolationOptions = {
  namespace?: string;
  debug?: boolean;
};

declare global {
  interface Window {
    __joliboxLocalStorage__: typeof window.localStorage;
  }
}

export function createStorageIsolation(options: StorageIsolationOptions = {}) {
  const namespace = options.namespace || getJoliMpId();
  const debug = options.debug || true;
  const observedFrames = new WeakSet<HTMLIFrameElement>();

  function getJoliMpId() {
    const urlParams = new URLSearchParams(window.location.search);
    return urlParams.get('mpId') ?? urlParams.get('appId') ?? urlParams.get('gameId') ?? '';
  }

  function _log(message: string) {
    if (debug) {
      console.log(`[StorageIsolation] ${message}`);
    }
  }

  function createIsolatedStorage() {
    const originalStorage = window.localStorage;

    return {
      setItem(key: string, value: string) {
        const namespacedKey = key.startsWith(`${namespace}:`) ? key : `${namespace}:${key}`;
        originalStorage.setItem(namespacedKey, value);
      },
      getItem(key: string) {
        const namespacedKey = key.startsWith(`${namespace}:`) ? key : `${namespace}:${key}`;
        let value = originalStorage.getItem(namespacedKey);
        if (value === null) {
          // 如果命名空间不存在，则尝试直接获取原始键
          const originalKey = key.startsWith(`${namespace}:`) ? key.slice(namespace.length + 1) : key;
          value = originalStorage.getItem(originalKey);
        }
        return value;
      },
      removeItem(key: string) {
        const namespacedKey = key.startsWith(`${namespace}:`) ? key : `${namespace}:${key}`;
        originalStorage.removeItem(namespacedKey);
      },
      clear() {
        const keys = Object.keys(originalStorage);
        for (const key of keys) {
          if (key.startsWith(`${namespace}:`)) {
            originalStorage.removeItem(key);
          }
        }
      },
      key(index: number) {
        const keys = Object.keys(originalStorage)
          .filter((key) => key.startsWith(`${namespace}:`))
          .map((key) => key.slice(namespace.length + 1));
        return keys[index] || null;
      },
      get length() {
        return Object.keys(originalStorage).filter((key) => key.startsWith(`${namespace}:`)).length;
      }
    };
  }

  function injectIsolatedStorage(
    frameWindow: Window,
    isolatedStorage: ReturnType<typeof createIsolatedStorage>
  ) {
    try {
      const originalStorage = frameWindow.localStorage;
      window.__joliboxLocalStorage__ = originalStorage;

      type StorageProp = keyof typeof isolatedStorage;
      const storageProxy = new Proxy(originalStorage, {
        get: (target, prop: StorageProp) => {
          if (prop in isolatedStorage) {
            return isolatedStorage[prop];
          }
          if (typeof target[prop] === 'function') {
            return target[prop].bind(target);
          }
          return target[prop];
        },
        set: (target, prop: StorageProp, value) => {
          if (prop === 'length') {
            return true;
          }
          if (prop in isolatedStorage) {
            isolatedStorage[prop] = value;
            return true;
          }
          target[prop] = value;
          return true;
        }
      });

      Object.defineProperty(frameWindow, 'localStorage', {
        get: () => storageProxy,
        configurable: true
      });
    } catch (e) {
      _log(`Failed to inject storage proxy: ${e}`);
    }
  }

  function injectIsolation(frame: HTMLIFrameElement) {
    try {
      const frameWindow = frame.contentWindow;
      const frameDocument = frame.contentDocument;

      if (!frameWindow || !frameDocument) {
        _log('Cannot access frame content - likely due to same-origin policy');
        return;
      }

      const isolatedStorage = createIsolatedStorage();
      injectIsolatedStorage(frameWindow, isolatedStorage);
      _log(`Successfully isolated storage for frame: ${frame.src}`);
    } catch (e) {
      _log(`Injection failed: ${e}`);
    }
  }

  function isolateFrame(frame: HTMLIFrameElement) {
    if (observedFrames.has(frame)) {
      return;
    }

    try {
      frame.addEventListener('load', () => {
        injectIsolation(frame);
      });
      if (frame.contentDocument && frame.contentDocument.readyState === 'complete') {
        injectIsolation(frame);
      }

      observedFrames.add(frame);
    } catch (e) {
      _log(`Failed to isolate frame: ${e}`);
    }
  }

  function handleExistingFrames() {
    document.querySelectorAll('iframe').forEach((frame) => {
      injectIsolation(frame);
    });
  }

  function startObserving() {
    handleExistingFrames();

    const observer = new MutationObserver((mutations) => {
      mutations.forEach((mutation) => {
        mutation.addedNodes.forEach((node) => {
          if (node instanceof HTMLIFrameElement) {
            isolateFrame(node);
          }
        });
      });
    });

    observer.observe(document.documentElement, {
      childList: true,
      subtree: true
    });

    _log('Started observing iframe creation');
  }

  function isolateWindowStorage() {
    const isolatedStorage = createIsolatedStorage();
    injectIsolatedStorage(window, isolatedStorage);
  }

  // Initialize
  startObserving();
  isolateWindowStorage();

  // Return public API if needed
  return {
    isolateFrame,
    isolateWindowStorage
  };
}

createStorageIsolation();
